type Query = Record<string, any>;
type Body = URLSearchParams | FormData | Query;
type ApiResponse<T, Safe extends boolean> = Promise<Safe extends true ? [T, Error] : T>;
interface Client<Safe extends boolean> {
    call<T>(method: string, url: string, qs?: Query, body?: Body): ApiResponse<T, Safe>;
    get<T>(url: string, qs?: Query, body?: Body): ApiResponse<T, Safe>;
    post<T>(url: string, qs?: Query, body?: Body): ApiResponse<T, Safe>;
    delete<T>(url: string, qs?: Query, body?: Body): ApiResponse<T, Safe>;
    postBody<T>(url: string, body: Body): ApiResponse<T, Safe>;
    put<T>(url: string, body: Body): ApiResponse<T, Safe>;
}
export interface Visitor {
    ok: <T>(response: Response) => Promise<T>;
    fail: <T>(request: Request, response: Response) => Promise<T>;
}
export interface CredentialsManager {
    applyRefresh: () => Promise<boolean>;
    isValid: (req: Request, res: Response) => Promise<boolean>;
    attach: (init: RequestInit, url?: string) => RequestInit | Promise<RequestInit>;
}
type ClientFactory = {
    visitor: Visitor;
    credentials: CredentialsManager;
};
export declare function addQueryParamsToUrl(url: string, queryParams?: Query): string;
export declare function createApiClient<Safe extends boolean>(factory: ClientFactory, safe: Safe): Client<Safe>;
export {};
